# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import math

pi = 3.14159

def convertrot(din):
    out = (2) * ( math.acos(din) * 180/pi )
    return out

# Custom functions
def objtype(btype):
    if btype == 1:
        return "pyramid\n"
    elif btype == 2:
        return "teleporter\n"
    else:
        return "box\n"

def objname(bname):
    objstr = "    name {}\n".format(name)
    return objstr

def objposition(x, y, z):
    objstr = "    position {} {} {}\n".format(x, y, z)
    return objstr

def objsize(btype, x, y, z):
    if btype == 2:
        objsize = "    size .125 {} {}\n    border 1\nend\n\n".format(y, z)
    else:
        objsize = "    size {} {} {}\nend\n\n".format(x, y, z)
    return objsize

def makeobject(otype ,oname, opos, orot : float, osize):
    if orot != 0.0: 
        bzwobjs = "{}{}{}{}".format(otype, oname, opos, osize)
    else:
        bzwobjs = "{}{}{}    rotation {}\n{}".format(otype, oname, opos, orot, osize)
    return bzwobjs

# For parsing numbers
def getF1(s1):
    l1 = s1.find(" ")
    l1 += 1
    s2 = s1[l1:]
    return s2

# get text between non symmetrical quotes
def getasymmquotes(text, start, end):
    findstartnum = text.find(start)
    findaddnum = len(start)
    startnum = findstartnum + findaddnum
    endnum = text.find(end)
    textwithin = text[startnum:endnum]
    return textwithin

def gotoend(msg):
    print(msg)
    sys.exit()

if __name__ == '__main__':
    print("Anim8or to BZW converter.")
    print("Licensed under GNUv3. (see license.txt)")
    print("Made by EvanEjk.")
    print("Ported to Python by Zehra")
    print("=====================")
    ask = input("Proceed? (1=yes 0=no)")
    askint = float(int(ask))
    if askint != 1:
        sys.exit("Close program.")
    else:
        try:
            fileIn = open("map.an8", "r")
        except:
            sys.exit("Error reading file.")
        
        print("Anim8or file loaded.")
        try:
            fileOut = open("map.bzw", "w")
        except:
            sys.exit("Error opening file.")
    fileOut.write("#Converted from anim8or file.\n#Converter made by EvanEjk.\n#Ported to Python by Zehra\n\n")
    
    tasklevel = 0
    # vars
    bzwType = 0 #By default it is a box.
    #Types: 0 = box, 1 = pyr, 2 = tel
    rotation = 0.0
    xPos = 0.0
    yPos = 0.0
    zPos = 0.0
    xSize = 0.0
    ySize = 0.0
    zSize = 0.0
    
    lines = fileIn.read()
    for line in lines.splitlines():
        if tasklevel == 0:
            bzwType = 0 #By default it is a box.
            #Types: 0 = box, 1 = pyr, 2 = tel
            rotation = 0.0
            xPos = 0.0
            yPos = 0.0
            zPos = 0.0
            xSize = 0.0
            ySize = 0.0
            zSize = 0.0
            found = line.find("  cube {")
            if found > -1:
                print("********** Found an object. **********")
                tasklevel = 1
        elif tasklevel == 1:
            found = line.find("name") #find the name!
            if found > -1:
                tasklevel = 2
                print("Found the object's name line.")
                print(line)
                fPos= line.find("\"") #finds first quote mark
                fPos += 1
                linevar = line[fPos:]
                fPos2 = linevar.find("\"") #finds second quote mark
                length = fPos + fPos2 
                name = line[fPos:length] #copies the name into name
                print("Name: {}".format(name))
                # find object type
                pyrtest = line.find("pyr")
                if pyrtest > -1:
                    print("Object is a pyramid.")
                    bzwType = 1
                teltest = line.find("tel")
                if teltest > -1:
                    print("Object is a teleport.")
                    bzwType = 2
        elif tasklevel == 2:
            found = line.find("origin")
            if found > -1:
                tasklevel = 3
                print("Found the origin line.")
                print(line)
                linevar = getasymmquotes(line, "(", ")")
                # Get coordinates/positions
                data = linevar.split(" ")
                xPos = float(data[0])
                print("x position: {}".format(xPos))
                zPos = float(data[1])
                print("z position: {}".format(zPos))
                yPos = float(data[2])
                print("y position: {}".format(yPos))
        elif tasklevel == 3:
            found = line.find("orientation")
            findscale = line.find("scale")
            if found > -1:
                #tasklevel = 4
                print("Found the orientation line.")
                linevar = getasymmquotes(line, "(", ")")
                data = linevar.split(" ")
                rotString = data[3]
                print("orientation: {}".format(rotString))
                rotDouble = float(rotString)
                rotation = convertrot(rotDouble)
            else:
                if findscale > -1:
                    print("No orientation.")
                    #tasklevel = 4
            if findscale > -1:
                #tasklevel = 5
                print("Found scale (size) line.")
                print(line)
                linevar = getasymmquotes(line, "{ ", " }")
                # get sizes
                data = linevar.split(" ")
                xSize = float(data[0])
                print("x size: {}".format(xSize))
                zSize = float(data[1])
                print("z size: {}".format(zSize))
                ySize = float(data[2])
                print("y size: {}".format(ySize))
        #elif tasklevel == 5:
                tasklevel = 0
                print("** Fixing bzflag's stupid object size to cordinate ratio. **")
                zPos += zPos
                xPos += xPos
                yPos += yPos
                print("** Fixing bzflag's stupid height ratio. **")
                zSize += zSize
                print("** Fixing bzflag's stupid object height/position thing... **")
                zPos = zPos - (.5 * zSize)
                print("** Fixing bzflag's stupid Y pos/neg flip thing... **\n\n")
                yPos = yPos * -1
                #get object settings
                bobj = objtype(bzwType)
                bname = objname(name)
                bpos = objposition(xPos, yPos, zPos)
                bsize = objsize(bzwType, xSize, ySize, zSize)
                # make object
                bobject = makeobject(bobj, bname, bpos, rotation, bsize)
                fileOut.write(bobject)
       

    if tasklevel == 1:
        gotoend("Error while finding object's name.")
    elif tasklevel == 2:
        gotoend("Error while finding origin.")
    elif tasklevel == 3:
        gotoend("Error: while finding orientation.")
    elif tasklevel == 4:
        gotoend("Error while finding scale (size).")
    else:
        print("\n*** At end of file. ***")

    fileIn.close()
    fileOut.close()
    sys.exit("Done.")
